//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* I create new Vectors, BoundingBox2 and other objects when i could pre-compute it
	* There are a few number constants that should be declaired as constants
	* Screen width and height needs to be GLOBAL!
*/

package starship2d;


import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;


public class Map extends Object
{
	private static final int NUM_OBJECTS = 7;
	private static final float OBJ_SIZE = 100;
	private static final int NUM_STARFIELDS = 3;
	// FIXME! make these global somehow
	private static final int SCR_WIDTH = 640;
	private static final int SCR_HEIGHT = 480;
	
	private int textures[] = new int[2];
	private float rot = 0;
	private MapBase objects[];

	Map()
	{
	}
	
	public void init()
	{
		textures[0] = Texture.loadTexture("starship2d/Data/rock.png");
		textures[1] = Texture.loadTexture("starship2d/Data/rockmask.png");
		
		objects = new MapBase[NUM_OBJECTS];
		
		for (int i=0; i < NUM_STARFIELDS; i++)
		{
			objects[i] = new MapStarfield(100,60*(i+1), new Vector3(0.4f*(i+1), 0.4f*(i+1), 0.4f*(i+1)));
		}
		
		for (int i=NUM_STARFIELDS; i < objects.length; i++)
		{
			objects[i] = new MapObject(textures);
			generateNewObject(i);
		}
	}
	
	public boolean render()
	{
		GL11.glColor3f(1.0f,1.0f,1.0f);
		for (int i=0; i < objects.length; i++)
		{
			objects[i].render();
		}
		
		return true;
	}
	
	public void update()
	{
		for (int i=0; i < objects.length; i++)
		{
			if (!objects[i].update() && i >= NUM_STARFIELDS)
			{
				generateNewObject(i);
			}
		}
	}
	
	public void destroy()
	{
	}
	
	private void generateNewObject(int i)
	{
		((MapObject)objects[i]).setPos(new Vector2(Math.random() * -(SCR_WIDTH*2.0f) + (SCR_WIDTH*1.5f), -OBJ_SIZE));
		((MapObject)objects[i]).setSize(new Vector2(OBJ_SIZE,OBJ_SIZE));
		((MapObject)objects[i]).setVelocity(new Vector2((Math.random() * 300 + 200) * ((Math.random() > 0.5) ? -1 : 1), 
														(Math.random() * 300 + 200)));
		((MapObject)objects[i]).setColor(new Vector3(Math.random(), Math.random(), Math.random()));
	}
	
	public boolean hit(BoundingBox2 bb) { return false; }
	public boolean sphereHit(float sqrRad, Vector2 center) { return false; }

} // end Map

abstract class MapBase
{
	public abstract void render();
	public abstract boolean update();
}

class MapObject extends MapBase
{
	// VARIABLES
	// FIXME! make these global somehow
	private static final int SCR_WIDTH = 640;
	private static final int SCR_HEIGHT = 480;

	private int textures[] = new int[2];
	private Vector2 pos;
	private Vector2 size;
	private Vector2 vel;
	private Vector3 color;
		
	// FUNCTIONS
	MapObject(int txt[])
	{
		textures[0] = txt[0];
		textures[1] = txt[1];
	}
	
	public void setPos(Vector2 newPos)		{ pos = newPos; }
	public void setSize(Vector2 newSize)	{ size = newSize; }
	public void setVelocity(Vector2 newVel)	{ vel = newVel; }
	public void setColor(Vector3 col)		{ color = col; }
	
	public void render()
	{
		GL11.glPushMatrix();
		
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glTranslatef(pos.x,pos.y,0);
		
		// draw the mask
		GL11.glBlendFunc(GL11.GL_DST_COLOR,GL11.GL_ZERO);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, textures[1]);
		GL11.glBegin(GL11.GL_QUADS);
			GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
			GL11.glTexCoord2f(0,1);	GL11.glVertex2f(0, size.y);
			GL11.glTexCoord2f(1,1);	GL11.glVertex2f(size.x, size.y);
			GL11.glTexCoord2f(1,0);	GL11.glVertex2f(size.x, 0);
		GL11.glEnd();
		
		// draw the texture
		GL11.glColor3f(color.x, color.y, color.z);
		
		GL11.glBlendFunc(GL11.GL_ONE, GL11.GL_ONE);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, textures[0]);
		GL11.glBegin(GL11.GL_QUADS);
			GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
			GL11.glTexCoord2f(0,1);	GL11.glVertex2f(0, size.y);
			GL11.glTexCoord2f(1,1);	GL11.glVertex2f(size.x, size.y);
			GL11.glTexCoord2f(1,0);	GL11.glVertex2f(size.x, 0);
		GL11.glEnd();
		
		GL11.glColor3f(1.0f,1.0f,1.0f);
		
		GL11.glDisable(GL11.GL_BLEND);

		GL11.glPopMatrix();
	}
	
	public boolean update()
	{
		if ( ( (pos.y - size.y) > SCR_WIDTH ) || ( pos.x -2*size.x > SCR_HEIGHT ) || ( pos.x  < -SCR_HEIGHT) )
		{
			return false;
		}
		
		pos.x += Timer.frameInterval * vel.x;
		pos.y += Timer.frameInterval * vel.y;
		
		
		return true;
	}	
}

class MapStarfield extends MapBase
{
	private static final int SCR_WIDTH = 640;
	private static final int SCR_HEIGHT = 480;
	private Vector2 stars[];
	private Vector3 color;
	private int numStars = 0;
	private float speed = 0.0f;
	
	MapStarfield(int nstars, float newSpeed, Vector3 vcolor)
	{
		color = vcolor;
		speed = newSpeed;
		numStars = nstars;
		stars = new Vector2[numStars];
		
		for (int i=0; i < numStars; i++)
			stars[i] = new Vector2(Math.random() * SCR_WIDTH, Math.random() * SCR_HEIGHT);
	}

	public void setSpeed(float newSpeed)	{ speed = newSpeed; }
	
	public void render()
	{
		GL11.glPushMatrix();
		
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		GL11.glColor3f(color.x, color.y, color.z);
		GL11.glPointSize(2.0f);
		for (int i=0; i < numStars; i++)
		{
			GL11.glBegin(GL11.GL_POINTS);
				GL11.glVertex2f(stars[i].x, stars[i].y);
			GL11.glEnd();
		}
		GL11.glPointSize(1.0f);
		GL11.glColor3f(1.0f,1.0f,1.0f);
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		
		GL11.glPopMatrix();
	}
	
	public boolean update()
	{
		for (int i=0; i < numStars; i++)
		{
			stars[i].y += Timer.frameInterval * speed;
			
			if (stars[i].y > SCR_HEIGHT)
			{
				stars[i] = new Vector2(Math.random() * SCR_WIDTH, 0);
			}
		}
		
		return true;
	}
}
