//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* I create new Vectors, BoundingBox2 and other objects when i could pre-compute it
	* There are a few number constants that should be declaired as constants
	* Screen width and height needs to be GLOBAL!
*/

package starship2d;

import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;

public class Rocket
{
	// VARIABLES
	private int scrWidth = 640;
	private int scrHeight = 480;

	private int txtRocket, txtMask;
	private Vector2 pos = new Vector2(100,100);
	private Vector2 v = new Vector2(0.0f,-0.1f);
	private Vector2 size =  new Vector2(40,40);
	private float decSpeed = 0;
	private ParticleRocket pRocket = new ParticleRocket();
	private ParticleExplosion pExplosion = new ParticleExplosion(100);
	private boolean isHit = false;
	private boolean isVisible = true;
	private Layer layer;
	private Player player;		// only used for coillision detection, we don't want to hit the player
	private float expSqrRadius;

	// FUNCTIONS
	public Rocket(Layer lay, Player pl)
	{
		layer = lay;
		player = pl;
		txtRocket = Texture.loadTexture("starship2d/Data/rocket.png");
		txtMask = Texture.loadTexture("starship2d/Data/rocketmask.png");

		pRocket.init(50,"starship2d/Data/particle.png");
		pRocket.setTailProp(13.0f,0.1f);

		expSqrRadius = 150*150;
	}
	
	public void setPos(Vector2 p)
	{ 
		pos = new Vector2(p); 
		pRocket.setCurPos(new Vector2(pos.x + 28, pos.y + 30));
	}
	public void setVelocity(Vector2 vel)	{ v =  new Vector2(v); }
	public void setSize(Vector2 s)			{ size = new Vector2(s); }
	public void setDecSpeed(float dec)		{ decSpeed = dec; }
	public void setExpSqrRadius(float r)	{ expSqrRadius = r; }

	public void reset()
	{
		isHit = false;
		isVisible = true;

		pRocket.setCurVel(v);
		pRocket.setCurPos(new Vector2(pos.x + 28, pos.y + 30));
		pRocket.reInit();
	}

	// return false if it should be destroyd
	public boolean render()
	{
		boolean res = true;
		GL11.glEnable(GL11.GL_BLEND);

		// pRocket should always be rendered
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA,GL11.GL_ONE);
		res = pRocket.render();

		if (isHit)
		{
			// we have to add || res here so it won't just die whent he explision
			// is done, it should wait for the pRocket too
			res = pExplosion.render() || res;
		}
		else if (isVisible)
		{
			GL11.glPushMatrix();

			GL11.glTranslatef(pos.x,pos.y,0);

			// draw the mask
			GL11.glBlendFunc(GL11.GL_DST_COLOR,GL11.GL_ZERO);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtMask);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(0, size.y);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x, size.y);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x, 0);
			GL11.glEnd();
			
			// draw the texture
			GL11.glBlendFunc(GL11.GL_ONE, GL11.GL_ONE);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtRocket);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(0, size.y);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x, size.y);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x, 0);
			GL11.glEnd();
			
			GL11.glPopMatrix();

			pos.x += v.x * (Timer.frameInterval * 500);
			pos.y += v.y * (Timer.frameInterval * 500);
			v.x -= v.x * decSpeed * (Timer.frameInterval * 300);
			v.y -= v.y * decSpeed * (Timer.frameInterval * 300);
			
			if ( (pos.y + size.y) < 0)
			{
				isVisible = false;
				pRocket.setRegenerate(false);
			}
			
			pRocket.setCurVel(v);
			pRocket.setCurPos(new Vector2(pos.x + 28, pos.y + 30));
			
			res = true;
		}
		
		GL11.glDisable(GL11.GL_BLEND);
		
		// check collision
		// because the rocket texture is only 17 pizel wide in a 64 pixel wide texture
		// we need to compensate the size of the rocket
		BoundingBox2 bb = new BoundingBox2(new Vector2(pos.x + size.x*(1 -(17.0f/64.0f)), pos.y),
											new Vector2(size.x * (17.0f/64.0f), size.y));

		if (!isHit && layer.checkCollision(bb,player))
		{
			isHit = true;

			pExplosion.init(1.0f, 0.5f, 0.5f, 0, new Vector2(pos.x + size.x/2 - 5, pos.y + size.y/2 - 5),
							new Vector2(20,20), v);

			pRocket.setRegenerate(false);

			// do a sphere collision too, so the rocket get more power
			layer.checkSphereCollision(expSqrRadius,pos, player);
		}
		
		return res;
	}
	
} // end Rocket
