//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* create a LWJGL class that takes care of all the window creation and
	that keeps the window size so all objects can access it

	* rename this class
*/

package starship2d;


import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.nio.FloatBuffer;

import org.lwjgl.LWJGLException;
import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;


public class StarShip2d
{
	// VARIABLES
	private boolean done = false;
	private boolean fullscreen = false;
	private final String windowTitle = "StarShip2D";
	private Layer layers[] = new Layer[2];
	private GameMenu menu;

	// FUNCTIONS
	public static void main(String[] args)
	{
		boolean fullscreen = false;
		if(args.length>0)
		{
			if(args[0].equalsIgnoreCase("fullscreen"))
			{
				fullscreen = true;
			}
		}

		StarShip2d t = new StarShip2d();
		t.run(fullscreen);
	}

	public void run(boolean fullscreen)
	{
		this.fullscreen = fullscreen;
		try
		{
			init();
			while (!done)
			{
				mainloop();
				render();
				Window.update();
			}
			cleanup();
        }
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(0);
		}
	}

	private void mainloop()
	{
		Keyboard.poll();

		if(Keyboard.isKeyDown(Keyboard.KEY_ESCAPE))
			done = true;

		if(Window.isCloseRequested())
            done = true;
	}

	private void switchMode()
	{
		Window.destroy();
		fullscreen = !fullscreen;

		try
		{
			createWindow();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(0);
		}

		initGL();
    }

	private void createWindow() throws Exception
	{
		if (fullscreen)
		{
			DisplayMode d[] = Display.getAvailableDisplayModes();
			int mode = 0;
			for (int i = 0; i < d.length; i++)
			{

				if (d[i].width == 640
					&& d[i].height == 480
					&& d[i].bpp == 32)
				{
					mode = i;
					break;
				}
			}
			Window.create(windowTitle, 32, 0, 8, 0);
		}
		else
			Window.create(windowTitle, 10, 10, 640, 480, 16, 0, 8, 0);
    }

    private void init()
	{
		System.runFinalizersOnExit(true);

		try
		{
			createWindow();
			Keyboard.create();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(0);
		}
		
		initGL();
		
		// init the timer with an update
		Timer.init();
		Timer.updateTimer();
		
		menu = new GameMenu(null, this);
		initLayers();
		
    }

	public void initLayers()
	{
		// remove referense to old layers
		// so the garbage collector eats them
		layers[0] = null;
		layers[1] = null;
		
		layers[0] = new Layer();
		layers[1] =  new Layer();
		
		Player p = new Player(layers[1], menu);

		layers[0].add(new Map());
		layers[0].init();
		layers[1].add(p);
		layers[1].add(new EnemyFormations(layers[1], p));
		layers[1].init();

		menu.setPlayer(p);
		menu.setFont(p.getFont());
		menu.reset();
	}
	
	private void initGL()
	{
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		GL11.glShadeModel(GL11.GL_SMOOTH);
		GL11.glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
		GL11.glClearDepth(1.0f);

		setOrtho();
	}

	private void setPerspective()
	{
		GL11.glMatrixMode(GL11.GL_PROJECTION);
		GL11.glLoadIdentity();
		GLU.gluPerspective(45.0f, (float) Display.getWidth() / (float) Display.getHeight(), 0.1f, 100.0f);
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glEnable(GL11.GL_DEPTH_TEST);
	}

	private void setOrtho()
	{
		GL11.glMatrixMode(GL11.GL_PROJECTION);
		GL11.glLoadIdentity();
		GL11.glOrtho(0, 640, 480, 0, -100, 100);
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glDisable(GL11.GL_DEPTH_TEST);
	}

	private void cleanup()
	{
		Keyboard.destroy();
		Window.destroy();
	}

	private void render()
	{
		Timer.updateTimer();
		
		if (!menu.isActive())
		{
			for (int i=0; i < layers.length; i++)
			{
				layers[i].update();
			}
		}
		
		GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
		GL11.glLoadIdentity();

		for (int i=0; i < layers.length; i++)
		{
			layers[i].render();
		}
		
		menu.render();
	}
	
} // End test
